ISO_wine_producers = [
    "IT",
    "ES",
    "FR",
    "US",
    "CN",
    "AR",
    "CL",
    "AU",
    "ZA",
    "DE",
    "PT",
    "RO",
    "GR",
    "RU",
    "NZ",
    "BR",
    "HU",
    "AT",
    "RS",
    "MD",
    "BG",
    "GE",
    "CH",
    "UA",
    "JP",
    "PE",
    "UY",
    "CA",
    "DZ",
    "CZ",
    "MK",
    "HR",
    "TR",
    "MX",
    "TM",
    "MA",
    "UZ",
    "SK",
    "BY",
    "AL",
    "KZ",
    "TN",
    "ME",
    "LB",
    "SI",
    "CO",
    "LU",
    "CU",
    "EE",
    "CY",
    "AZ",
    "BO",
    "MG",
    "BA",
    "AM",
    "LT",
    "EG",
    "IL",
    "BE",
    "LV",
    "MT",
    "ZW",
    "KG",
    "PY",
    "ET",
    "JO",
    "GB",
    "PA",
    "TJ",
    "LI",
    "SY",
    "PL",
    "RE",
]

# country object - will have ISO code, basic name, all names, nationality forms, and whether it's a wine producer
